#!/bin/sh

#
# AirVPN suite installation script
#
# Version 2.1.0 - 9 December 2024
#
# Written by ProMIND
#

get_bin_path()
{
    BIN_PATH="/bin /sbin /usr/bin /usr/sbin /usr/local/bin /usr/local/sbin"

    local CMD_PATH=$2
    eval $CMD_PATH=$1

    for CPATH in $BIN_PATH; do
        TBIN="${CPATH}/${1}"

        if [ -f ${TBIN} ]; then
            eval $CMD_PATH=$TBIN
        fi
    done
}

get_dbus_path()
{
    TPATH="/etc/dbus-1/system.d /usr/local/etc/dbus-1/system.d /usr/share/dbus-1/system.d /usr/local/share/dbus-1/system.d"

    for DPATH in $TPATH; do
        if [ -d ${DPATH} ] && [ "${DBUS_PATH}" = "" ]; then
            DBUS_PATH=$DPATH
        fi
    done
}

get_bin_path useradd useradd_bin
get_bin_path groupadd groupadd_bin
get_bin_path usermod usermod_bin
get_bin_path passwd passwd_bin

DBUS_PATH=""

get_dbus_path

AIRVPN_USER=airvpn
AIRVPN_GROUP=airvpn
TS_CAP=cap_sys_admin,cap_net_admin,cap_net_raw
CAP_FILE=/etc/security/capability.conf

echo
echo "AirVPN suite installation script"
echo

if [ `id -u` != "0" ]; then
    echo "You need root privilegs to install AirVPN suite"
    echo

    exit 1
fi

read -p "Do you want to install AirVPN Suite? [y/n] " yn

if [ "${yn}" = "n" ]; then
    echo
    echo "Installation aborted"

    exit 1
fi

INIT_TYPE="unknown"

echo

if [ `cat < /proc/1/comm` = "systemd" ]; then
    INIT_TYPE=systemd
    echo "System is using systemd"
elif [ `cat < /proc/1/comm` = "init" ]; then
    INIT_TYPE=init
    echo "System is using SysV-style init"
else
    echo "This system is not running either systemd or SysV-style init"
    echo "Installation aborted"
    echo

    exit 1
fi

if [ "${DBUS_PATH}" = "" ]; then
    echo
    echo "ERROR: D-Bus is not properly configured or not available"
    echo "Installation aborted"
    echo

    exit 1
else
    echo
    echo "D-Bus directory is ${DBUS_PATH}"
    echo
fi

if [ "${INIT_TYPE}" = "systemd" ]; then
    systemctl --quiet is-active bluetit.service

    if [ $? -eq 0 ]; then
        systemctl stop bluetit.service

        sleep 5
    fi

    systemctl --quiet is-active bluetit-suspend.service

    if [ $? -eq 0 ]; then
        systemctl stop bluetit-suspend.service
    fi

    systemctl --quiet is-active bluetit-resume.service

    if [ $? -eq 0 ]; then
        systemctl stop bluetit-resume.service
    fi
else
    /etc/init.d/bluetit stop 2&>1 > /dev/null
fi

echo "Installing bluetit to /sbin"

cp bin/bluetit /sbin/bluetit

if [ -d "/usr/local/bin" ]; then
    BIN_DIR=/usr/local/bin
elif [ -d "/usr/bin" ]; then
    BIN_DIR=/usr/bin
else
    BIN_DIR=/bin
fi

echo "Installing goldcrest to ${BIN_DIR}"

cp bin/goldcrest $BIN_DIR

echo "Installing hummingbird to ${BIN_DIR}"

cp bin/hummingbird $BIN_DIR

echo "Installing cuckoo to ${BIN_DIR}"

cp bin/cuckoo $BIN_DIR

echo "Installing airsu to ${BIN_DIR}"

cp bin/airsu $BIN_DIR

echo "Installing bluetit configuration files"

if [ ! -d "/etc/airvpn" ]; then
    mkdir /etc/airvpn
fi

chmod 750 /etc/airvpn

cp etc/airvpn/airvpn-manifest.xml /etc/airvpn
cp etc/airvpn/connection_priority.txt /etc/airvpn
cp etc/airvpn/connection_sequence.csv /etc/airvpn
cp etc/airvpn/country_continent.csv /etc/airvpn
cp etc/airvpn/country_names.csv /etc/airvpn
cp etc/airvpn/continent_names.csv /etc/airvpn
cp etc/airvpn/nsswitch.conf /etc/airvpn

if [ ! -f "/etc/airvpn/bluetit.rc" ]; then
    cp etc/airvpn/bluetit.rc /etc/airvpn
fi

chmod 660 -R /etc/airvpn

echo "Installing D-Bus configuration files"

cp etc/dbus-1/system.d/* $DBUS_PATH

chmod 644 $DBUS_PATH/org.airvpn.*

if [ "${INIT_TYPE}" = "systemd" ]; then
    # systemd

    echo "Installing systemd bluetit.service units"

    cp etc/systemd/system/bluetit.service /etc/systemd/system
    cp etc/systemd/system/bluetit-suspend.service /etc/systemd/system
    cp etc/systemd/system/bluetit-resume.service /etc/systemd/system

    systemctl daemon-reload

    echo

    read -p "Do you want to enable bluetit.service units? [y/n] " yn

    if [ "${yn}" = "y" ]; then
        systemctl enable bluetit.service

        if [ $? -eq 0 ]; then
            echo "Bluetit service enabled"
        else
            echo "Cannot enable Bluetit service"
            echo "Run 'systemctl status bluetit.service' for more information"
            echo

            exit 1
        fi

        systemctl enable bluetit-suspend.service

        if [ $? -eq 0 ]; then
            echo "Bluetit-suspend service enabled"
        else
            echo "Cannot enable Bluetit-suspend service"
            echo "Run 'systemctl status bluetit-suspend.service' for more information"
            echo

            exit 1
        fi

        systemctl enable bluetit-resume.service

        if [ $? -eq 0 ]; then
            echo "Bluetit-resume service enabled"
        else
            echo "Cannot enable Bluetit-resume service"
            echo "Run 'systemctl status bluetit-resume.service' for more information"
            echo

            exit 1
        fi
    fi

    echo

    read -p "Do you want to start Bluetit service now? [y/n] " yn

    if [ "${yn}" = "y" ]; then
        systemctl stop bluetit.service
        systemctl daemon-reload
        systemctl start bluetit.service

        if [ $? -eq 0 ]; then
            echo "Bluetit service started"
        else
            echo "Cannot start Bluetit service"
            echo "Run 'systemctl status bluetit.service' for more information"
            echo

            exit 1
        fi
    fi

else
    # SysV-style init

    echo "Installing bluetit SysV-style init script"

    cp etc/init.d/bluetit /etc/init.d

    echo

    read -p "Do you want to start bluetit at boot? [y/n] " yn

    if [ "${yn}" = "y" ]; then
        which chkconfig > /dev/null 2>&1

        if [ $? -eq 0 ]; then
            chkconfig --add bluetit
        else
            ln -s /etc/init.d/bluetit /etc/rc0.d/K60bluetit
            ln -s /etc/init.d/bluetit /etc/rc1.d/K60bluetit
            ln -s /etc/init.d/bluetit /etc/rc2.d/K60bluetit
            ln -s /etc/init.d/bluetit /etc/rc3.d/S90bluetit
            ln -s /etc/init.d/bluetit /etc/rc4.d/K60bluetit
            ln -s /etc/init.d/bluetit /etc/rc5.d/S90bluetit
            ln -s /etc/init.d/bluetit /etc/rc6.d/K60bluetit
        fi
    fi

    echo

    read -p "Do you want to start Bluetit daemon now? [y/n] " yn

    if [ "${yn}" = "y" ]; then
        /etc/init.d/bluetit start

        if [ $? -eq 0 ]; then
            echo "Bluetit service started"
        else
            echo "Cannot start Bluetit daemon"
            echo "Run 'cat < /var/log/syslog | grep bluetit' for more information"
            echo

            exit 1
        fi
    fi

fi

id $AIRVPN_USER > /dev/null 2>&1

if [ $? -ne 0 ]; then
    echo

    read -p "Do you want to create ${AIRVPN_USER} user? [y/n] " yn

    if [ "${yn}" = "y" ]; then
        $useradd_bin -m $AIRVPN_USER

        if [ $? -eq 0 ]; then
            echo "Please set a password for user ${AIRVPN_USER}"

            $passwd_bin $AIRVPN_USER
        fi
    fi
fi

grep -q ^$AIRVPN_GROUP /etc/group

if [ $? -ne 0 ]; then
    echo

    read -p "Do you want to create ${AIRVPN_GROUP} group? [y/n] " yn

    if [ "${yn}" = "y" ]; then
        $groupadd_bin $AIRVPN_GROUP

        if [ $? -ne 0 ]; then
            echo "ERROR: Cannot create group ${AIRVPN_GROUP}"
            echo

            exit 1
        fi
    fi
fi

id $AIRVPN_USER > /dev/null 2>&1

if [ $? -eq 0 ]; then
    grep -q ^$AIRVPN_GROUP /etc/group

    if [ $? -eq 0 ]; then
        grep ^$AIRVPN_GROUP /etc/group | cut -d: -f4 | grep -q $AIRVPN_USER

        if [ $? -eq 1 ]; then
            $usermod_bin -a -G $AIRVPN_GROUP $AIRVPN_USER > /dev/null 2>&1

            if [ $? -eq 0 ]; then
                echo

                echo "User ${AIRVPN_USER} added to group ${AIRVPN_GROUP}"
            else
                echo "ERROR: Cannot add user ${AIRVPN_USER} to group ${AIRVPN_GROUP}"
                echo

                exit 1
            fi
        fi
    fi
fi

# auto completion scripts

if [ -d "/usr/share/bash-completion/completions" ]; then
    echo "Installing auto completion scripts for bash"

    cp etc/bash-completion/* /usr/share/bash-completion/completions
fi

if [ -d "/usr/share/zsh/site-functions" ]; then
    echo "Installing auto completion scripts for zsh"

    cp etc/site-functions/* /usr/share/zsh/site-functions
fi

# finalize cuckoo and airvpn permissions and settings

chown root:airvpn $BIN_DIR/cuckoo

chmod u+s $BIN_DIR/cuckoo

setcap cap_net_raw,cap_sys_admin,cap_net_admin=ep $BIN_DIR/cuckoo

if [ ! -f ${CAP_FILE} ]; then
    touch ${CAP_FILE}
fi

grep -qx "${TS_CAP}.*${AIRVPN_USER}" ${CAP_FILE}

if [ $? -ne 0 ]; then
    echo
    echo "${TS_CAP} ${AIRVPN_USER}" >> ${CAP_FILE}
    echo
    echo "${TS_CAP} @${AIRVPN_GROUP}" >> ${CAP_FILE}
fi

if [ "${INIT_TYPE}" = "systemd" ]; then
    systemctl reload dbus.service
else
    /etc/init.d/dbus reload
fi

echo "Done."
