#!/bin/sh

#
# AirVPN suite uninstallation script
#
# Version 2.0.0 - 9 December 2024
#
# Written by ProMIND
#

get_bin_path()
{
    BIN_PATH="/bin /sbin /usr/bin /usr/sbin /usr/local/bin /usr/local/sbin"

    local CMD_PATH=$2
    eval $CMD_PATH=$1

    for CPATH in $BIN_PATH; do
        TBIN="${CPATH}/${1}"

        if [ -f ${TBIN} ]; then
            eval $CMD_PATH=$TBIN
        fi
    done
}

get_dbus_path()
{
    TPATH="/etc/dbus-1/system.d /usr/local/etc/dbus-1/system.d /usr/share/dbus-1/system.d /usr/local/share/dbus-1/system.d"

    for DPATH in $TPATH; do
        TCONF="${DPATH}/org.airvpn.server.conf"

        if [ -f ${TCONF} ] && [ "${DBUS_PATH}" = "" ]; then
            DBUS_PATH=$DPATH
        fi
    done
}

get_bin_path userdel userdel_bin
get_bin_path groupdel groupdel_bin

DBUS_PATH=""

get_dbus_path

AIRVPN_USER=airvpn
AIRVPN_GROUP=airvpn
TS_CAP=cap_sys_admin,cap_net_admin,cap_net_raw
CAP_FILE=/etc/security/capability.conf

if [ ! -f "/sbin/bluetit" ]; then
    echo
    echo "AirVPN Suite is not installed in this system"
    echo

    exit 1
fi

echo
echo "AirVPN Suite uninstallation script"
echo

if [ `id -u` != "0" ]; then
    echo "You need root privilegs to uninstall AirVPN Suite"
    echo

    exit 1
fi

read -p "Do you want to completely uninstall AirVPN Suite? [y/n] " yn

if [ "${yn}" = "n" ]; then
    echo
    echo "Uninstallation aborted"

    exit 1
fi

INIT_TYPE="unknown"

echo

if [ `cat < /proc/1/comm` = "systemd" ]; then
    INIT_TYPE=systemd
    echo "System is using systemd"
elif [ `cat < /proc/1/comm` = "init" ]; then
    INIT_TYPE=init
    echo "System is using SysV-style init"
else
    echo "This system is not running either systemd or SysV-style init"
    echo "Installation aborted"
    echo

    exit 1
fi

if [ "${DBUS_PATH}" = "" ]; then
    echo
    echo "ERROR: D-Bus is not properly configured or not available"
    echo "Uninstallation aborted"
    echo

    exit 1
else
    echo
    echo "D-Bus directory is ${dbuspath}"
    echo
fi

if [ "${INIT_TYPE}" = "systemd" ]; then
    # systemd

    if [ -f "/etc/systemd/system/bluetit.service" ]; then
        echo "Stopping bluetit.service"

        systemctl stop bluetit.service

        echo "Disabling bluetit.service"

        systemctl disable bluetit.service

        echo "Removing systemd bluetit.service unit"

        rm /etc/systemd/system/bluetit.service

        echo "Disabling bluetit-suspend.service"

        systemctl disable bluetit-suspend.service

        echo "Removing systemd bluetit-suspend.service unit"

        rm /etc/systemd/system/bluetit-suspend.service

        echo "Disabling bluetit-resume.service"

        systemctl disable bluetit-resume.service

        echo "Removing systemd bluetit-resume.service unit"

        rm /etc/systemd/system/bluetit-resume.service

        systemctl daemon-reload
    fi
else
    # SysV-style init

    if [ -f "/etc/init.d/bluetit" ]; then
        echo "Stopping bluetit daemon"

        /etc/init.d/bluetit stop

        which chkconfig  > /dev/null 2>&1

        if [ $? -eq 0 ]; then
            chkconfig --del bluetit
        else
            rm /etc/rc0.d/K60bluetit
            rm /etc/rc1.d/K60bluetit
            rm /etc/rc2.d/K60bluetit
            rm /etc/rc3.d/S90bluetit
            rm /etc/rc4.d/K60bluetit
            rm /etc/rc5.d/S90bluetit
            rm /etc/rc6.d/K60bluetit
        fi

        echo "Removing SysV-style bluetit init script"

        rm /etc/init.d/bluetit
    fi
fi

echo "Removing D-Bus configuration files"

if [ -f "${DBUS_PATH}/org.airvpn.client.conf" ]; then
    rm $DBUS_PATH/org.airvpn.client.conf
fi

if [ -f "${DBUS_PATH}/org.airvpn.server.conf" ]; then
    rm $DBUS_PATH/org.airvpn.server.conf
fi

which hummingbird  > /dev/null 2>&1

if [ $? -eq 0 ]; then
    echo "Removing hummingbird"
    rm `which hummingbird`
fi

which goldcrest  > /dev/null 2>&1

if [ $? -eq 0 ]; then
    echo "Removing goldcrest"
    rm `which goldcrest`
fi

which cuckoo  > /dev/null 2>&1

if [ $? -eq 0 ]; then
    echo "Removing cuckoo"
    rm `which cuckoo`
fi

which airsu  > /dev/null 2>&1

if [ $? -eq 0 ]; then
    echo "Removing airsu"
    rm `which airsu`
fi

if [ -f "/sbin/bluetit" ]; then
    echo "Removing bluetit"
    rm /sbin/bluetit
fi

if [ -d "/etc/airvpn" ]; then
    echo "Removing bluetit configuration files"
    rm /etc/airvpn/*

    echo "Removing /etc/airvpn directory"
    rmdir /etc/airvpn
fi

id $AIRVPN_USER > /dev/null 2>&1

if [ $? -eq 0 ]; then
    echo

    read -p "Do you want to delete ${AIRVPN_USER} user and its home directory? [y/n] " yn

    if [ "${yn}" = "y" ]; then
        $userdel_bin -r $AIRVPN_USER
    fi
fi

grep -q $AIRVPN_GROUP /etc/group

if [ $? -eq 0 ]; then
    echo

    read -p "Do you want to delete ${AIRVPN_GROUP} group? [y/n] " yn

    if [ "${yn}" = "y" ]; then
        $groupdel_bin $AIRVPN_GROUP
    fi
fi

echo

if [ -f "/usr/share/bash-completion/completions/goldcrest" ]; then
    rm /usr/share/bash-completion/completions/goldcrest
fi

if [ -f "/usr/share/bash-completion/completions/hummingbird" ]; then
    rm /usr/share/bash-completion/completions/hummingbird
fi

if [ -f "/usr/share/bash-completion/completions/cuckoo" ]; then
    rm /usr/share/bash-completion/completions/cuckoo
fi

if [ -f "/usr/share/bash-completion/completions/airsu" ]; then
    rm /usr/share/bash-completion/completions/airsu
fi

if [ -f "/usr/share/zsh/site-functions/_goldcrest" ]; then
    rm /usr/share/zsh/site-functions/_goldcrest
fi

if [ -f "/usr/share/zsh/site-functions/_hummingbird" ]; then
    rm /usr/share/zsh/site-functions/_hummingbird
fi

if [ -f "/usr/share/zsh/site-functions/_cuckoo" ]; then
    rm /usr/share/zsh/site-functions/_cuckoo
fi

if [ -f "/usr/share/zsh/site-functions/_airsu" ]; then
    rm /usr/share/zsh/site-functions/_airsu
fi

sed -i "/${TS_CAP}.*${AIRVPN_USER}/d" ${CAP_FILE}

if [ "${INIT_TYPE}" = "systemd" ]; then
    systemctl reload dbus.service
else
    /etc/init.d/dbus reload
fi

echo "Done."
